import xml.etree.ElementTree as ET
from datetime import date
from decimal import Decimal

import pytest

from ofx_generator.generator import OFXGenerator
from ofx_generator.models import (
    Balance,
    BankAccount,
    BankStatement,
    CreditCardAccount,
    FinancialInstitution,
    OFXSettings,
    Transaction,
)
from ofx_generator.standards import AccountType, Currency, Language, TransactionType


@pytest.fixture
def sample_statement():
    fi = FinancialInstitution(org="Test Bank", fid="123")
    account = BankAccount(
        bank_id="123",
        branch_id="0001",
        account_id="12345",
        account_type=AccountType.CHECKING,
    )
    transaction = Transaction(
        transaction_type=TransactionType.CREDIT,
        date_posted=date(2024, 1, 1),
        amount=Decimal("100.00"),
        fit_id="123",
        memo="Test transaction",
    )
    balance = Balance(amount=Decimal("1000.00"), date_as_of=date(2024, 1, 1))
    settings = OFXSettings(language=Language.ENGLISH, currency=Currency.US_DOLLAR, version=211)
    credit_card_account = CreditCardAccount(
        cash_advance_available_amount=500.00, cash_advance_credit_limit=2000.00
    )
    return BankStatement(
        financial_institution=fi,
        bank_account=account,
        transactions=[transaction],
        balance=balance,
        start_date=date(2024, 1, 1),
        end_date=date(2024, 1, 31),
        settings=settings,
        credit_card_account=credit_card_account,
    )


def test_generator_initialization():
    generator = OFXGenerator()
    assert generator is not None


def test_ofx_generation(sample_statement):
    generator = OFXGenerator()
    trnuid = "test-trnuid-123"
    ofx_content = generator.generate(sample_statement, trnuid)

    print("Generated OFX Content:")
    print(ofx_content)

    # Get XML content after headers
    xml_lines = ofx_content.split("\n")
    xml_content = "\n".join(xml_lines[2:])

    # Verify it's valid XML
    root = ET.fromstring(xml_content)

    # Define the namespace map
    namespaces = {"ofx": "http://ofx.net/ifx/2.0/ofx"}

    # Check basic structure with namespace
    assert root.tag == "{http://ofx.net/ifx/2.0/ofx}OFX"
    assert root.find(".//ofx:SIGNONMSGSRSV1", namespaces) is not None
    assert root.find(".//ofx:BANKMSGSRSV1", namespaces) is not None

    # Check financial institution info
    assert root.find(".//{*}FI/{*}ORG").text == "Test Bank"
    assert root.find(".//{*}FI/{*}FID").text == "123"

    # Check transaction
    transaction = root.find(".//{*}STMTTRN")
    assert transaction is not None
    assert transaction.find("{*}TRNAMT").text == "100.00"
    assert transaction.find("{*}MEMO").text == "Test transaction"

    # Check balance
    assert root.find(".//{*}LEDGERBAL/{*}BALAMT").text == "1000.00"

    # Check TRNUID
    assert root.find(".//{*}TRNUID").text == trnuid


def test_ofx_headers(sample_statement):
    generator = OFXGenerator()
    trnuid = "test-trnuid-123"
    ofx_content = generator.generate(sample_statement, trnuid)

    # Check headers
    headers = ofx_content.split("\n")
    assert '<?xml version="1.0" encoding="UTF-8" standalone="no"?>' == headers[0]
    assert (
        '<?OFX OFXHEADER="200" VERSION="230" SECURITY="NONE" '
        'OLDFILEUID="NONE" NEWFILEUID="NONE"?>' == headers[1]
    )


def test_credit_card_account():
    account = CreditCardAccount(cash_advance_available_amount=0.0, cash_advance_credit_limit=0.0)
    assert account.cash_advance_available_amount == 0.0  # Test for new field
    assert account.cash_advance_credit_limit == 0.0  # Test for new field



from setuptools import find_packages, setup

# Read the version from the version file
version = {}
with open("ofx_generator/__version__.py") as fp:
    exec(fp.read(), version)

setup(
    name="ofx-generator",
    version=version["__version__"],
    author="Marcos Bressan",
    author_email="bressan@dee.ufc.br",
    description="A library for generating OFX files.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/bressanmarcos/ofx-generator",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



from datetime import date
from decimal import Decimal

import pytest

from ofx_generator.models import (
    Balance,
    BankAccount,
    BankStatement,
    FinancialInstitution,
    OFXSettings,
    Transaction,
    ValidationError,
)
from ofx_generator.standards import AccountType, Currency, Language, TransactionType


@pytest.fixture
def valid_fi():
    return FinancialInstitution(org="Test Bank", fid="123")


@pytest.fixture
def valid_account():
    return BankAccount(
        bank_id="123",
        branch_id="0001",
        account_id="12345",
        account_type=AccountType.CHECKING,
    )


@pytest.fixture
def valid_transaction():
    return Transaction(
        transaction_type=TransactionType.CREDIT,
        date_posted=date(2024, 1, 1),
        amount=Decimal("100.00"),
        fit_id="123",
        memo="Test transaction",
    )


@pytest.fixture
def valid_balance():
    return Balance(amount=Decimal("1000.00"), date_as_of=date(2024, 1, 1))


@pytest.fixture
def valid_settings():
    return OFXSettings(language=Language.ENGLISH, currency=Currency.US_DOLLAR, version=211)


def test_financial_institution_creation():
    fi = FinancialInstitution(org="Test Bank", fid="123")
    assert fi.org == "Test Bank"
    assert fi.fid == "123"


def test_bank_account_creation():
    account = BankAccount(
        bank_id="123",
        branch_id="0001",
        account_id="12345",
        account_type=AccountType.CHECKING,
    )
    assert account.bank_id == "123"
    assert account.branch_id == "0001"
    assert account.account_id == "12345"
    assert account.account_type == AccountType.CHECKING


def test_transaction_creation():
    transaction = Transaction(
        transaction_type=TransactionType.CREDIT,
        date_posted=date(2024, 1, 1),
        amount=Decimal("100.00"),
        fit_id="123",
        memo="Test transaction",
    )
    assert transaction.transaction_type == TransactionType.CREDIT
    assert transaction.amount == Decimal("100.00")
    assert transaction.fit_id == "123"


def test_balance_creation():
    balance = Balance(amount=Decimal("1000.00"), date_as_of=date(2024, 1, 1))
    assert balance.amount == Decimal("1000.00")
    assert balance.date_as_of == date(2024, 1, 1)


def test_bank_statement_creation(
    valid_fi, valid_account, valid_transaction, valid_balance, valid_settings
):
    statement = BankStatement(
        financial_institution=valid_fi,
        bank_account=valid_account,
        transactions=[valid_transaction],
        balance=valid_balance,
        start_date=date(2024, 1, 1),
        end_date=date(2024, 1, 31),
        settings=valid_settings,
    )
    assert statement.financial_institution == valid_fi
    assert statement.bank_account == valid_account
    assert len(statement.transactions) == 1
    assert statement.balance == valid_balance


def test_invalid_transaction_amount():
    with pytest.raises(ValidationError):
        Transaction(
            transaction_type=TransactionType.CREDIT,
            date_posted=date(2024, 1, 1),
            amount=Decimal("0"),  # Invalid amount (should be non-zero)
            fit_id="123",
            memo="Test transaction",
        )


def test_invalid_date_range(
    valid_fi, valid_account, valid_transaction, valid_balance, valid_settings
):
    with pytest.raises(ValidationError):
        BankStatement(
            financial_institution=valid_fi,
            bank_account=valid_account,
            transactions=[valid_transaction],
            balance=valid_balance,
            start_date=date(2024, 1, 31),  # End date before start date
            end_date=date(2024, 1, 1),
            settings=valid_settings,
        )
